@@ -37,6 +37,22 @@ size_t Chunk::Length() const noexcept {
 	return end - start;
 }
 
+namespace {
+
+#ifdef __APPLE__
+// Archaic macOS colours for the default: black on light yellow
+constexpr ColourRGBA colourTextAndArrow(black);
+constexpr ColourRGBA colourBackground(0xff, 0xff, 0xc6);
+#else
+// Grey on white
+constexpr ColourRGBA colourTextAndArrow(0x80, 0x80, 0x80);
+constexpr ColourRGBA colourBackground(white);
+#endif
+
+constexpr ColourRGBA silver(0xc0, 0xc0, 0xc0);
+
+}
+
 CallTip::CallTip() noexcept {
 	wCallTip = {};
 	inCallTipMode = false;
@@ -54,17 +70,12 @@ CallTip::CallTip() noexcept {
 	borderHeight = 2; // Extra line for border and an empty line at top and bottom.
 	verticalOffset = 1;
 
-#ifdef __APPLE__
-	// proper apple colours for the default
-	colourBG = ColourRGBA(0xff, 0xff, 0xc6);
-	colourUnSel = ColourRGBA(0, 0, 0);
-#else
-	colourBG = ColourRGBA(0xff, 0xff, 0xff);
-	colourUnSel = ColourRGBA(0x80, 0x80, 0x80);
-#endif
+	colourBG = colourBackground;
+	colourUnSel = colourTextAndArrow;
+
 	colourSel = ColourRGBA(0, 0, 0x80);
-	colourShade = ColourRGBA(0, 0, 0);
-	colourLight = ColourRGBA(0xc0, 0xc0, 0xc0);
+	colourShade = black;
+	colourLight = silver;
 	codePage = 0;
 	clickPlace = 0;
 }
@@ -108,14 +119,14 @@ void DrawArrow(Surface *surface, const PRectangle &rc, bool upArrow, ColourRGBA
 
 	constexpr XYPOSITION pixelMove = 0.0f;
 	if (upArrow) {      // Up arrow
-		Point pts[] = {
+		const Point pts[] = {
 			Point(centreX - halfWidth + pixelMove, centreY + quarterWidth + 0.5f),
 			Point(centreX + halfWidth + pixelMove, centreY + quarterWidth + 0.5f),
 			Point(centreX + pixelMove, centreY - halfWidth + quarterWidth + 0.5f),
 		};
 		surface->Polygon(pts, std::size(pts), FillStroke(colourBG));
 	} else {            // Down arrow
-		Point pts[] = {
+		const Point pts[] = {
 			Point(centreX - halfWidth + pixelMove, centreY - quarterWidth + 0.5f),
 			Point(centreX + halfWidth + pixelMove, centreY - quarterWidth + 0.5f),
 			Point(centreX + pixelMove, centreY + halfWidth - quarterWidth + 0.5f),
@@ -272,7 +283,7 @@ void CallTip::MouseClick(Point pt) noexcept {
 }
 
 PRectangle CallTip::CallTipStart(Sci::Position pos, Point pt, int textHeight, const char *defn,
-                                 int codePage_, Surface *surfaceMeasure, std::shared_ptr<Font> font_) {
+                                 int codePage_, Surface *surfaceMeasure, const std::shared_ptr<Font> &font_) {
 	clickPlace = 0;
 	val = defn;
 	codePage = codePage_;
